//Wrap the whole mess in the following function to make it work in Drupal
(function($) {

  $(document).ready(function () {

    // Original tablecollapse.js functionality
    // Initialize table to fadeOut
    $('.toggle-table').fadeIn();

    // Bind Expand-button method
    $('.collapse-table-button').click(() => $('.toggle-table').fadeOut());

    // Bind the toggle button method
    $('.toggle-button').click(() =>
      $(this).parent().find("table").fadeToggle( "2000", "linear" ));

    // Copy buttons
    const copySvg = '<svg focusable="false" aria-hidden="true" viewBox="0 0 24 24"><path d="M16 1H4c-1.1 0-2 .9-2 2v14h2V3h12V1zm3 4H8c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h11c1.1 0 2-.9 2-2V7c0-1.1-.9-2-2-2zm0 16H8V7h11v14z"></path></svg>';
    const checkSvg = '<svg focusable="false" aria-hidden="true" viewBox="0 0 24 24"><path d="M9 16.17 4.83 12l-1.42 1.41L9 19 21 7l-1.41-1.41z"></path></svg>';

    // Copy button: Collect elem's text and put it on the clipboard
    function elemTextToClipboard(elem) {
      let range = document.createRange();
      range.selectNode(elem);
      window.getSelection().removeAllRanges();
      window.getSelection().addRange(range);
      document.execCommand("copy");
      window.getSelection().removeAllRanges();
    }

    // Copy button: Add the copy button to elem, must have position: relative
    function addCopyButtonTo(elem) {
      const $elem = $(elem);
      let $cb = $("<button>");
      $cb[0].innerHTML = copySvg;
      $cb.addClass("highlight-json-copy-button");
      $cb.attr("title", "Copy to clipboard");
      $elem.prepend($cb);   // Add button to pre
      $cb.on("click", (evt) => {
        $cb.addClass("confirm");
        $cb[0].innerHTML = checkSvg;
        elemTextToClipboard(elem);
        evt.stopPropagation();
        evt.preventDefault();
        setTimeout(() => {
          $cb.removeClass("confirm");
          $cb[0].innerHTML = copySvg;
        }, 1000);
      });
    }

    // Table-column sorting: sort $table on column $th, asc true for ascending
    function sortRows($table, $th, asc) {
      let colIndex = $th.attr("data-col-index");
      if (!colIndex) return;
      let $tbody = $table.find("> tbody");
      let rows = $tbody.find("> tr").remove();
      try {
        rows.sort((a, b) => {
          const va = $(a).find("td")[colIndex].textContent;
          const vb = $(b).find("td")[colIndex].textContent;
          return asc ?vb.localeCompare(va) : va.localeCompare(vb);
        });
      } catch(ex) {}
      $tbody.append(rows);

      // To sort the table, we remove the rows from the document, sort them,
      // then put them back. This is orders of magnitude faster than sorting
      // the rows while they are still in the DOM. The downside is that any
      // nested tables will lose their event handlers up there at .remove().
      // Here we reinstall event handlers for any nested tables.
      installTableSorting($tbody);
    }

    // Table-column sorting functionality. This also works for nested tables.
    function installTableSorting($fromHere) {
      if (!$fromHere)
        $fromHere = $("div[role='main']");
      const $tables = $fromHere.find("table.docutils");
      $tables.each((i, table) => {
        const $table = $(table);
        const $ths = $(table).find("> thead > tr > th");
        $ths.each((j, th) => {
          const $th = $(th);
          $th.addClass("sortable");
          $th.attr("data-col-index", j);
          const $p = $th.find("> p");
          $p.on("click", evt => {
            const $targ = $(evt.target);
            const $th = $targ.closest("th");
            const asc = $targ.hasClass("ascending");
            const desc = $targ.hasClass("descending");
            const $headParas = $targ.closest("thead")
              .find("th > p")
              .removeClass("ascending descending");
            $headParas.removeClass("ascending descending");
            if (desc || (!asc && !desc)) {
              $targ.removeClass("descending");
              $targ.addClass("ascending");
              sortRows($table, $th, asc);
            } else if (asc) {
              $targ.removeClass("ascending");
              $targ.addClass("descending");
              sortRows($table, $th, asc);
            }
          });
        });
      });
    }

    // Code-collapse functionality
    // Here we look into the content to see how big it is, and if it's small, we
    // auto-expanded the div. All .highlight-json divs still have the
    // expand/collapse functionality - we just pre-expand the small ones.
    var kCollapseThresholdBytes = 1024;
    $(".highlight-json").each(function(index, elem)
    {
      const $elem = $(elem);
      if ($elem.text().length < kCollapseThresholdBytes)
        $elem.addClass("expanded");
      $elem.on("click", (evt) =>
        $(evt.target).closest(".highlight-json").toggleClass("expanded"));
      addCopyButtonTo($elem[0]);
    });
    $(".highlight-default").each(function(index, elem)
    {
      const $elem = $(elem);
      if ($elem.text().length < kCollapseThresholdBytes)
        $elem.addClass("expanded");
      $elem.on("click", (evt) =>
        $(evt.target).closest(".highlight-default").toggleClass("expanded"));
      addCopyButtonTo($elem[0]);
    });
    // Table-column sorting functionality
    installTableSorting();

  }); // document.ready()

})(jQuery);
